@@ -55,8 +55,11 @@ class AutocompleteEntry(hildon.Entry, ):
         self._dict = ['abc', 'moja mama', 'hahahahaha', 'haloooo', 'ca']
         self.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL)
         self._handler = self.connect('insert-text', self.on_insert_text)
+        self.connect('key-press-event', self.on_key_press_event)
         self._transform = None
         self._completed = False
+        self._ends = []
+        self._current = 0
     def set_dictionary(self, d):
         self._dict = d
     def _get_begin_of_word(self, text, pos, delimiters=None):
@@ -78,23 +81,46 @@ class AutocompleteEntry(hildon.Entry, ):
         return (''.join(begin).lstrip() if (len(begin) > 0) else None)
     def _find_end(self, begin):
         l = len(begin)
+        ends = []
         for item in self._dict:
             if (item[:l] == begin):
-                return item[l:]
-        return None
+                ends.append(item[l:])
+        return ends
     def on_insert_text(self, widget, new_text, new_text_length, position, data=None):
         self._completed = False
         pos = widget.get_position()
         begin = self._get_begin_of_word((widget.get_text() + new_text), pos)
         if begin:
-            end = self._find_end(begin)
-            if end:
+            self._ends = self._find_end(begin)
+            if (len(self._ends) > 0):
                 self._completed = True
-                end = (self._transform(end) if self._transform else end)
+                self._current = 0
+                end = (self._transform(self._ends[self._current]) if self._transform else self._ends[self._current])
                 self.handler_block(self._handler)
                 widget.insert_text(end, (pos + 1))
                 self.handler_unblock(self._handler)
                 gobject.idle_add(widget.select_region, (pos + 1), ((pos + len(end)) + 1))
+    def on_key_press_event(self, widget, event, data=None):
+        key = gtk.gdk.keyval_name(event.keyval)
+        if (key in ('Up', 'Down')):
+            if (self._completed and (len(self._ends) > 1)):
+                dir = (1 if (key == 'Up') else -1)
+                bounds = self.get_selection_bounds()
+                if ((len(bounds) == 2) and (dir != None)):
+                    self._current = ((self._current + dir) % len(self._ends))
+                    end = (self._transform(self._ends[self._current]) if self._transform else self._ends[self._current])
+                    pos = widget.get_position()
+                    self.handler_block(self._handler)
+                    text = widget.get_text()
+                    widget.set_text((text[:bounds[0]] + text[bounds[1]:]))
+                    widget.insert_text(end, (pos + 1))
+                    self.handler_unblock(self._handler)
+                    gobject.idle_add(widget.select_region, bounds[0], ((pos + len(end)) + 1))
+            return True
+    def downComplete(self):
+        pass
+    def upComplete(self):
+        pass
     def set_transform(self, cb):
         self._transform = cb
 
